@@ -1,6 +1,6 @@
 ;;; authors.el --- utility for maintaining Emacs's AUTHORS file  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2000-2022 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2023 Free Software Foundation, Inc.
 
 ;; Author: Gerd Moellmann <gerd@gnu.org>
 ;; Maintainer: emacs-devel@gnu.org
@@ -48,7 +48,9 @@ files.")
     ("Alexander Gramiak" "Alex Gramiak")
     ("Alexandru Harsanyi" "Alex Harsanyi")
     ("Álvar Jesús Ibeas Martín" "Álvar Ibeas")
+    (nil "ambihelical")
     ("Andrea Corallo" "AndreaCorallo")
+    ("Andrii Kolomoiets" "andreyk\\.mad@gmail\\.com")
     ("Andrew Csillag" "Drew Csillag")
     ("Andrew G Cohen" "Andrew Cohen")
     ("Anna M. Bigatti" "Anna Bigatti")
@@ -56,19 +58,26 @@ files.")
     ("Barry A. Warsaw" "Barry A. Warsaw, Century Computing, Inc."
      "Barry A. Warsaw, ITB" "Barry Warsaw")
     ("Bastien Guerry" "Bastien .*bzg")
+    ("Benjamin Carlsson" "glacials" "ben@twos\\.dev")
+    ("Benjamin Schwerdtner" "Benjamin\\.Schwerdtner@gmail\\.com" "Ben Schw")
     ("Bill Carpenter" "WJ Carpenter")
     ("Bill Mann" "William F. Mann")
     ("Bill Rozas" "Guillermo J. Rozas")
-    (nil "binjo.cn@gmail.com")
-    (nil "bug-gnu-emacs@gnu.org")  ; mistake
+    ("Billy Zheng" "vil963@gmail\\.com")
+    (nil "binjo\\.cn@gmail\\.com")
+    (nil "bug-gnu-emacs@gnu\\.org")  ; mistake
     ("Björn Torkelsson" "Bjorn Torkelsson")
     ("Brian Fox" "Brian J. Fox")
     ("Brian P Templeton" "BT Templeton")
     ("Brian Sniffen" "Brian T. Sniffen")
+    (nil "brotzeitmacher@gmail\\.com")
     (nil "castor@my-dejanews")
-    (nil "chengang31@gmail.com")
+    (nil "chengang31@gmail\\.com")
     (nil "chuntaro")
     ("Clément Pit-Claudel" "Clément Pit--Claudel")
+    (nil "Cristian" "crstml@libero\\.it")
+    ("Le Trung Dan" "daanturo@gmail\\.com" "Daanturo")
+    ("Daniel Freeman" "dannyfreeman")
     ("David Abrahams" "Dave Abrahams")
     ("David J. Biesack" "David Biesack")
     ("David De La Harpe Golden" "David Golden")
@@ -81,7 +90,7 @@ files.")
     (nil "dalanicolai")
     (nil "deech@deech")
     ("Deepak Goel" "D. Goel")
-    ("Earl Hyatt" "Earl" "ej32u@protonmail.com")
+    ("Earl Hyatt" "Earl" "ej32u@protonmail\\.com")
     ("Ed L. Cashin" "Ed L Cashin")
     ("Edward M. Reingold" "Ed\\(ward\\( M\\)?\\)? Reingold" "Reingold Edward M")
     ("Emilio C. Lopes" "Emilio Lopes")
@@ -90,18 +99,27 @@ files.")
     ("Etienne Prud’Homme" "Etienne Prud'Homme")
     ("Fabián Ezequiel Gallina" "Fabian Ezequiel Gallina" "Fabi.n E\\. Gallina")
     (nil "felix.*EmacsWiki")
+    (nil "felix\\.dick@web\\.de")
     (nil "foudfou")
+    ("Feraidoon Mehri"
+     "NightMachinary"
+     "rudiwillalwaysloveyou@gmail\\.com" "fifymehry@gmail\\.com")
     ("Francis Litterio" "Fran Litterio")
     ("Francis J. Wright" "Dr Francis J. Wright" "Francis Wright")
     ("François Pinard" "Francois Pinard")
     ("Francesco Potortì" "Francesco Potorti" "Francesco Potorti`")
     ("Frederic Pierresteguy" "Fred Pierresteguy")
     (nil "^FSF")
+    (nil "galeo")
+    ("Garid Zorigoo" "garid3000" "garidzorigoo@gmail\\.com")
     ("Gerd Möllmann" "Gerd Moellmann")
+    (nil "haqle314")
     ("Hallvard B. Furuseth" "Hallvard B Furuseth" "Hallvard Furuseth")
+    (nil "hokomo@airmail\\.cc" "hokomo")
     ("Hrvoje Nikšić" "Hrvoje Niksic")
     ("Ian Dunn" "^Ian D\\>")
     ;; lisp/org/ChangeLog.1 2010-11-11.
+    ("Ignacio Casso" "ignacio\\.decasso@imdea\\.org" "ignaciocasso@hotmail\\.com")
     (nil "immerrr")
     (nil "aaa bbb")
     (nil "Code Extracted") ; lisp/newcomment.el's "Author:" header
@@ -118,6 +136,8 @@ files.")
     ("Jérémie Courrèges-Anglas" "Jeremie Courreges-Anglas")
     ("Jeremy Bertram Maitin-Shepard" "Jeremy Maitin-Shepard")
     ("Jérémy Compostella" "Jeremy Compostella")
+    (nil "jgarte@" "jgart")
+    (nil "jicksaw@pm\\.me" "jicksaw")
     ("Jimmy Aguilar Mena" "Ergus")
     ("Johan Bockgård" "Johan Bockgard")
     ("John F. Carr" "John F Carr")
@@ -131,6 +151,7 @@ files.")
     ("Juan León Lahoz García" "Juan-Leon Lahoz Garcia")
     ("Jürgen Hötzel" "Juergen Hoetzel")
     (nil "k3tu0isui")
+    (nil "kby@tilde\\.team")
     ("K. Shane Hartman" "Shane Hartman")
     ("Kai Großjohann" "Kai Grossjohann")
     ("Karl Berry" "K. Berry")
@@ -140,8 +161,14 @@ files.")
     ("Kenichi Handa" "Ken'ichi Handa" "Kenichi HANDA" "K\\. Handa")
     ("Kevin Greiner" "Kevin J. Greiner")
     ("Kim F. Storm" "Kim Storm")
+    ;; The dash is a kludge, so this contributor is not ignored.
+    ("kobarity-" "kobarity@gmail\\.com")
+    ("Koen van Greevenbroek" "realcomplex" "koenvg@posteo\\.net")
     ("Kyle Jones" "Kyle E. Jones")
     ("Lars Magne Ingebrigtsen" "Lars Ingebrigtsen")
+    ("Laurence Warne" "lWarne" "laurencewarne@gmail\\.com")
+    (nil "lensplaysgames@gmail\\.com")
+    (nil "lorniu@gmail\\.com")
     (nil "LynX@bk.ru")
     (nil "lu4nx")
     ("Marcus G. Daniels" "Marcus Daniels")
@@ -163,14 +190,19 @@ files.")
     ("Michael R. Cook" "Michael Cook")
     ("Michael Sperber" "Mike Sperber" "Michael Sperber \\[Mr. Preprocessor\\]")
     ("Michalis V" "^mvar")
-    ("Miha Rihtaršič" "Miha Rihtarsic")
+    ("Miha Rihtaršič" "Miha Rihtarsic" "miha@kamnitnik\\.top" "miha")
     ("Mikio Nakajima" "Nakajima Mikio")
+    (nil "montag451@laposte\\.net")
+    (nil "na@aisrntairetnraoitn")
     ("Nelson Jose dos Santos Ferreira" "Nelson Ferreira")
     ("Noorul Islam" "Noorul Islam K M")
 ;;;    ("Tetsurou Okazaki" "OKAZAKI Tetsurou") ; FIXME?
+    (nil "odanoburu@")
+    (nil "one\\.last\\.kiss@outlook\\.com")
+    ("Cao ZhenXiang" "mail@ookami\\.one")
     ("Óscar Fuentes" "Oscar Fuentes")
     (nil "pillule")
-    (nil "psyberbits@gmail.com")
+    (nil "psyberbits@gmail\\.com")
     ("Paul Eggert" "Paul R\\. Eggert")
     ("Pavel Janík" "Pavel Janík Ml." "Pavel Janik Ml." "Pavel Janik")
     ("Pavel Kobiakov" "Pavel Kobyakov")
@@ -183,14 +215,18 @@ files.")
     ("Philipp Stephani" "Philipp .*phst@google")
     ("Piotr Zieliński" "Piotr Zielinski")
     ("Po Lu" "Po Lu Via")               ; looks like a mistake
+    ("Po Lu" "oldosfan" "luangruo@yahoo\\.com")
     ("Przemysław Wojnowski" "Przemyslaw Wojnowski")
-    ("R. Bernstein" "rb@dustyfeet.com")
+    ("Qifan Wang" "LdBeth" "andpuke@foxmail\\.com")
+    ("R. Bernstein" "rb@dustyfeet\\.com")
     ("Rainer Schöpf" "Rainer Schoepf")
     ("Raja R. Harinath" "Raja R Harinath")
     ("Rasmus Pank Roulund" "Rasmus .*rasmus@gmx")
+    (nil "rbrtb@")
     ("Richard G. Bielawski" "Richard G Bielawski" "Richard Bielawski")
     ("Richard King" "Dick King")
     ("Richard M. Stallman" "Richard Stallman" "rms@gnu.org")
+    (nil "webmaster@robario\\.com")
     ("Robert J. Chassell" "Bob Chassell")
     ("Roberto Huelga Díaz" "Roberto Huelga")
     ("Rodney J. Whitby" "Rod Whitby")
@@ -198,17 +234,19 @@ files.")
     ("Ron Schnell" "Ronnie Schnell")
     ("Rui-Tao Dong" "Rui-Tao Dong ~{6-HpLN~}")
     ("Ryan Thompson" "Ryan .*rct@thompsonclan")
+    (nil "rvs314")
     (nil "rzl24ozi")
     ("Sacha Chua" "Sandra Jean Chua")
     ("Sam Steingold" "Sam Shteingold")
     ("Satyaki Das" "Indexed search by Satyaki Das")
     ("Sébastien Vauban" "Sebastien Vauban")
     ("Sergey Litvinov" "Litvinov Sergey")
-    ("Simen Heggestøyl" "simenheg@gmail.com")
+    ("Simen Heggestøyl" "simenheg@gmail\\.com")
     (nil "prime.wizard")
     ("Shun-ichi Goto" "Shun-ichi GOTO")
     ;; There are other Stefans.
 ;;;    ("Stefan Monnier" "Stefan")
+    (nil "ssnnoo")
     ("Steven L. Baur" "SL Baur" "Steven L Baur")
     ("Stewart M. Clamen" "Stewart Clamen")
     ("Stuart D. Herring" "Stuart Herring" "Davis Herring")
@@ -222,6 +260,7 @@ files.")
     ("Thomas Dye" "Tom Dye")
     ("Thomas Horsley" "Tom Horsley")	; FIXME ?
     ("Thomas Wurgler" "Tom Wurgler")
+    (nil "thuna\\.cing@gmail\\.com")
     ("Toby Cubitt" "Toby S\\. Cubitt")
     ("Tomohiko Morioka" "MORIOKA Tomohiko")
     ("Torbjörn Axelsson" "Torbjvrn Axelsson")
@@ -230,23 +269,29 @@ files.")
     ("Tsugutomo Enami" "enami tsugutomo")
     ("Ulrich Müller" "Ulrich Mueller")
     (nil "vividsnow")
+    (nil "Valenoern" "valenoern@distributary\\.network")
     ("Vincent Del Vecchio" "Vince Del Vecchio")
+    ("M Visuwesh" "visuweshm@gmail\\.com")
+    (nil "vjoki@")
+    (nil "whatacold@gmail\\.com")
     ("William M. Perry" "Bill Perry")
     ("Wlodzimierz Bzyl" "W.*dek Bzyl")
     (nil "xyblor")
+    ("Yilkal Argaw" "yilkalargaw" "yilkalargawworkneh@gmail\\.com")
     ("Yoni Rabkin" "Yoni Rabkin Katzenell")
     ("Yoshinori Koseki" "KOSEKI Yoshinori" "小関 吉則")
+    ("Yuzhana Ego" "YugaEgo" "yet@ego\\.team")
     ("Yutaka NIIBE" "NIIBE Yutaka")
     (nil "stardiviner")
-    (nil "lin.sun")
+    (nil "lin\\.sun")
     ("Nitish Chinta" "nitishch")
     ("Carlos Pita" "memeplex")
     ("Vinicius Jose Latorre" "viniciusjl")
     ("Gaby Launay" "galaunay")
     ("Dick R. Chiang" "dickmao")
-    ("Lin Zhou" "georgealbert@qq.com")
-    (nil "yan@metatem.net")
-    (nil "gnu_lists@halloleo.hailmail.net")
+    ("Lin Zhou" "georgealbert@qq\\.com")
+    (nil "yan@metatem\\.net")
+    (nil "gnu_lists@halloleo\\.hailmail\\.net")
     )
   "Alist of author aliases.
 
@@ -397,6 +442,8 @@ Changes to files matching one of the regexps in this list are not listed.")
     "autogen/missing" "autogen"
     "autogen/copy_autogen" ; not generated, but trivial and now removed
     "dir_top"
+    ;; Imported into Emacs but externally maintained.
+    "publicsuffix.txt" "SKK-JISYO.L"
     ;; Only existed briefly, then renamed:
     "images/icons/allout-widgets-dark-bg"
     "images/icons/allout-widgets-light-bg"
@@ -918,9 +965,30 @@ Changes to files in this list are not listed.")
     "emacsclient.c" "etags.c" "hexl.c" "make-docfile.c" "movemail.c"
     "test-distrib.c" "testfile"
     "tpu-edt.doc"			; see below
+    ;; etc/images/gnus/
+    "etc/images/gnus/important.pbm"
+    "etc/images/gnus/receipt.pbm"
+    "etc/images/gnus/unimportant.pbm"
     "iso-swed.el"
+    ;; lisp/obsolete/
     "lisp/obsolete/vc-mcvs.el"
     "obsolete/vc-mcvs.el"
+    "lisp/obsolete/patcomp.el"
+    "lisp/obsolete/abbrevlist.el"
+    "lisp/obsolete/assoc.el"
+    "obsolete/assoc.el"
+    "lisp/obsolete/complete.el"
+    "lisp/obsolete/cust-print.el"
+    "lisp/obsolete/erc-hecomplete.el"
+    "lisp/obsolete/mailpost.el"
+    "obsolete/mailpost.el"
+    "lisp/obsolete/mouse-sel.el"
+    "lisp/obsolete/old-emacs-lock.el"
+    "obsolete/old-emacs-lock.el"
+    "lisp/obsolete/patcomp.el"
+    "lisp/obsolete/pc-select.el"
+    "lisp/obsolete/s-region.el"
+    "obsolete/pc-select.el"
     "nnwarchive.el"
     "nnultimate.el"
     "nnslashdot.el"
@@ -956,7 +1024,41 @@ Changes to files in this list are not listed.")
     "flymake-ui.el"
     "pinentry.el"
     "ledit.el"
-    "lmenu.el")
+    "lmenu.el"
+    "src/pgtkselect.h"
+    "test/manual/noverlay/many-errors.h"
+    "lisp/better-pixel-scroll.el"
+    "test/infra/default-gitlab-ci.yml"
+    "test/infra/test-jobs-generator.sh"
+    "org-install.el"
+    ;; use-package stuff removed after it was added
+    "etc/USE-PACKAGE-NEWS"
+    "lisp/use-package/bind-chord.el"
+    "lisp/use-package/use-package-chords.el"
+    "lisp/use-package/use-package-chords-tests.el"
+    "test/lisp/use-package/use-package-chords-tests.el"
+    ;; Eglot stuff removed after it was added
+    "README.mdown"
+    "README.md"
+    ".travis.yml"
+    ;; Tree-sitter stuff
+    "admin/notes/tree-sitter/html-manual/Parser_002dbased-Font-Lock.html"
+    "admin/notes/tree-sitter/html-manual/Parsing-Program-Source.html"
+    "admin/notes/tree-sitter/html-manual/build-manual.sh"
+    "admin/notes/tree-sitter/html-manual/Accessing-Node.html"
+    "admin/notes/tree-sitter/html-manual/Language-Definitions.html"
+    "admin/notes/tree-sitter/html-manual/Multiple-Languages.html"
+    "admin/notes/tree-sitter/html-manual/Parser_002dbased-Indentation.html"
+    "admin/notes/tree-sitter/html-manual/Pattern-Matching.html"
+    "admin/notes/tree-sitter/html-manual/Retrieving-Node.html"
+    "admin/notes/tree-sitter/html-manual/Tree_002dsitter-C-API.html"
+    "admin/notes/tree-sitter/html-manual/Using-Parser.html"
+    "admin/notes/tree-sitter/html-manual/manual.css"
+    "admin/notes/tree-sitter/build-module/build.sh"
+    "admin/notes/tree-sitter/build-module/batch.sh"
+    "doc/misc/gnus-coding.texi"
+    "gnus-coding.texi"
+    )
   "File names which are valid, but no longer exist (or cannot be found)
 in the repository.")
 
@@ -988,7 +1090,7 @@ in the repository.")
 ;; to how a file was mentioned in the respective ChangeLog.  It is
 ;; advisable to run a Grep command such as
 ;;
-;;   fgrep -R BASENAME . --include='ChangeLog*'
+;;   grep -F -R BASENAME . --include='ChangeLog*'
 ;;
 ;; where BASENAME is the old basename of the renamed file.  This will
 ;; show all the different reference forms of the file in the various
@@ -1335,6 +1437,153 @@ in the repository.")
     ("test/lisp/url/url-handlers-test.el" . "url-handlers-tests.el")
     ("test/src/dired-tests.el" . "dired-tests.el")
     (".dir-locals.el" . ".dir-locals.el")
+    ;; use-package files that were moved when use-package was added:
+    ("use-package.texi" . "use-package.texi")
+    ("use-package-core.el" . "use-package-core.el")
+    ("bind-key.el" . "use-package-bind-key.el")
+    ("use-package.el" . "use-package.el")
+    ("use-package-tests.el" . "use-package-tests.el")
+    ;; pgtk stuff which used incorrect file names
+    ("pgtkmenu.c" . "pgtkmenu.c")
+    ("pgtk-win.el" . "pgtk-win.el")
+    ("pgtkfns.c" . "pgtkfns.c")
+    ("pgtkterm.c" . "pgtkterm.c")
+    ("pgtkterm.h" . "pgtkterm.h")
+    ("../src/pgtkfns.c" . "pgtkfns.c")
+    ("../src/pgtkterm.c" . "pgtkterm.c")
+    ("../src/pgtkterm.h" . "pgtkterm.h")
+    ("../src/atimer.c" . "atimer.c")
+    ("../src/gtkutil.c" . "gtkutil.c")
+    ("../src/image.c" . "image.c")
+    ("../lisp/faces.el" . "faces.el")
+    ("../src/pgkterm.h" . "pgkterm.h")
+    ("pgkterm.c" . "pgkterm.c")
+    ("../src/emacsgtkfixed.c" . "emacsgtkfixed.c")
+    ("../src/xfaces.c" . "xfaces.c")
+    ("../src/pgtkgui.h" . "pgtkgui.h")
+    ("../src/dispextern.h" . "dispextern.h")
+    ("../src/menu.c" . "menu.c")
+    ("../lisp/net/browse-url.el" . "browse-url.el")
+    ;; miscellany
+    ("nsterm.m" . "nsterm.m")
+    ("jsonrpc.el" . "jsonrpc.el")
+    ("jsonrpc-tests.el" . "jsonrpc-tests.el")
+    ("jrpc.el" . "jsonrpc.el")
+    ("eldoc.el" . "eldoc.el")
+    ("lisp/progmodes/ts-mode.el" . "typescript-ts-mode.el")
+    ("icalendar-tests.el" . "icalendar-tests.el")
+    ("lisp/progmodes/css-ts-mode.el" . "css-mode.el")
+    ("lisp/erc/erc-tests.el" . "erc-tests.el")
+    ("lisp/erc/erc-scenarios-base-reconnect.el" . "erc-scenarios-base-reconnect.el")
+    ("test/lisp/erc-tests.el" . "erc-tests.el")
+    ("eglot.el" . "eglot.el")
+    ("eglot-tests.el" . "eglot-tests.el")
+    ("NEWS.md" . "EGLOT-NEWS")
+    ("test/lisp/comp-tests.el" . "comp-tests.el")
+    ("package-vc.el" . "package-vc.el")
+    ("package.el" . "package.el")
+    ("lisp/net/tramp-docker.el" . "tramp-container.el")
+    ("xterm.c" . "xterm.c")
+    ("lisp/osc.el" . "ansi-osc.el")
+    ("test/lisp/osc-tests.el" . "ansi-osc-tests.el")
+    ("lisp/ansi-osc.el" . "ansi-osc-tests.el")
+    ("test/lisp/thumbs-tests.el" . "thumbs-tests.el")
+    ("rmail.el" . "rmail.el")
+    ("window.el" . "window.el")
+    ("nsmenu.m" . "nsmenu.m")
+    ("nsfont.m" . "nsfont.m")
+    ("nsfns.m" . "nsfns.m")
+    ("src/nsterm.c" . "nsterm.m")
+    ("subr.el" . "subr.el")
+    ("test/lisp/image-dired-tests.el" . "image-dired-tests.el")
+    ("modus-themes.org" . "modus-themes.org")
+    ("emacs-authors-mode.el" . "emacs-authors-mode.el")
+    ("lisp/textmodes/etc-authors-mode.el" . "emacs-authors-mode.el")
+    ("bytecomp.el" . "bytecomp.el")
+    ("test/lisp/makesum-tests.el" . "makesum-tests.el")
+    ("rcirc.el" . "rcirc.el")
+    ("haiku_support.cc" . "haiku_support.cc")
+    ("gnus-art.el" . "gnus-art.el")
+    ("mh-mime.el" . "mh-mime.el")
+    ("terminal.c" . "terminal.c")
+    ("eudc.texi" . "eudc.texi")
+    ("gnus-search.el" . "gnus-search.el")
+    ("lisp/gnus-search.el" . "gnus-search.el")
+    ("ETAGS_good_1" . "ETAGS_good_1")
+    ("ETAGS_good_2" . "ETAGS_good_2")
+    ("ETAGS_good_3" . "ETAGS_good_3")
+    ("ETAGS_good_4" . "ETAGS_good_4")
+    ("ETAGS_good_5" . "ETAGS_good_5")
+    ("ETAGS_good_6" . "ETAGS_good_6")
+    ("test/lisp/eshell-em-script-tests.el" . "eshell-em-script-tests.el")
+    ("test/lisp/eshell-em-glob-tests.el" . "eshell-em-glob-tests.el")
+    ("lisp/eshell/esh-var-tests.el" . "esh-var-tests.el")
+    ("test/lisp/eshell/esh-var-test.el" . "esh-var-tests.el")
+    ("gnus-logic.el" . "gnus-logic.el")
+    ("sh-script.el" . "sh-script.el")
+    ("repeat.el" . "repeat.el")
+    ("files.el" . "files.el")
+    ("lisp/emacs-lisp/generate-file.el" . "generate-lisp-file.el")
+    ("pp.el" . "pp.el")
+    ("src/help-fns.el" . "help-fns.el")
+    ("print.c" . "print.c")
+    ("shell.el" . "shell.el")
+    ("xdisp.c" . "xdisp.c")
+    ("haikufns.c" . "haikufns.c")
+    ("haikuterm.c" . "haikuterm.c")
+    ("haikumenu.c" . "haikumenu.c")
+    ("haikufont.c" . "haikufont.c")
+    ("src/haiku_support.c" . "haiku_support.cc")
+    ("src/haiku_draw_support.c" . "haiku_draw_support.cc")
+    ("haiku-win.el" . "haiku-win.el")
+    ("elisp-mode.el" . "elisp-mode.el")
+    ("doc-view.el" . "doc-view.el")
+    ("src/lisp/net/rcirc.el" . "rcirc.el")
+    ("project.el" . "project.el")
+    ("emacsbug.el" . "emacsbug.el")
+    ("timefns.c" . "timefns.c")
+    ("xwidget.c" . "xwidget.c")
+    ("src/xwidget.el" . "xwidget.el")
+    ("lisp/net/lisp/net/tramp-sshfs.el" . "tramp-sshfs.el")
+    ("tramp-sudoedit.el" . "tramp-sudoedit.el")
+    ("test/lisp/mail/undigest.el" . "undigest-tests.el")
+    ("Activate.c" . "Activate.c")
+    ("quail.el" . "quail.el")
+    ("sed1v2.inp" . "sed1v2.inp")
+    ("ruby-parenless-call-arguments-indent.rb" . "ruby-parenless-call-arguments-indent.rb")
+    ("commands.texi" . "doc/lispref/commands.texi")
+    ("message.el" . "message.el")
+    ("lisp/debug-early.el" . "debug-early.el")
+    ("tabulated-list.el" . "tabulated-list.el")
+    ("mouse.el" . "mouse.el")
+    ("hi-lock.el" . "hi-lock.el")
+    ("man.el" . "man.el")
+    ("doc/emacs/frames.tex" . "frames.texi")
+    ("lisp/emacs-list/eieio-compat.el" . "eieio-compat.el")
+    ("epa.el" . "epa.el")
+    ("lisp/emacs-lisp/macroexpand.el" . "macroexp.el")
+    ("src/pixel-scroll.el" . "pixel-scroll.el")
+    ("test/lisp/mh-e/mh-utils.el" . "mh-utils-tests.el")
+    ("compile.el" . "compile.el")
+    ("compile-tests.el" . "compile-tests.el")
+    ("Makefile.in" . "Makefile.in")
+    ("Makefie.in" . "Makefile.in")
+    ("test/lisp/net/netrc-tests.el" . "auth-source-tests.el")
+    ("test/lisp/ert-x-tests.el" . "ert-x-tests.el")
+    ("lisp/mh-e-mh-scan.el" . "mh-scan.el")
+    ("lisp/progmodes/c-fonts.el" . "cc-fonts.el")
+    ("lisp/emacs/lisp/cl-generic.el" . "cl-generic.el")
+    ("doc/lisprefdisplay.texi" . "display.texi")
+    ("erc.el" . "erc.el")
+    ("erc-tests.el" . "erc-tests.el")
+    ("vc/vc-mtn.el" . "vc-mtn.el")
+    ("net/rlogin.el" . "rlogin.el")
+    ("emacs-lisp/eieio-compat.el" . "eieio-compat.el")
+    ("mh-compat.el" . "mh-compat.el")
+    ("url-about.el" . "url-about.el")
+    ("url-dired.el" . "url-dired.el")
+    ("lisp/text-modes/tex-mode.el" . "tex-mode.el")
+    ("editfns.c" . "editfns.c")
     )
   "Alist of files which have been renamed during their lifetime.
 Elements are (OLDNAME . NEWNAME).")
@@ -1881,7 +2130,7 @@ list of their contributions.\n")
 		(insert "\n "))
 	    (insert " " file))
 	  (insert "\n")))))
-    (insert "\nLocal" " Variables:\nmode: etc-authors\ncoding: "
+    (insert "\nLocal" " Variables:\nmode: emacs-authors\ncoding: "
 	    (symbol-name authors-coding-system) "\nEnd:\n")
     (message "Generating buffer %s... done" buffer-name)
     (unless noninteractive
