@@ -1,6 +1,6 @@
-;;; calc-bin.el --- binary functions for Calc
+;;; calc-bin.el --- binary functions for Calc  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1990-1993, 2001-2020 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2023 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
 
@@ -126,8 +126,8 @@
 (defun calc-word-size (n)
   (interactive "P")
   (calc-wrapper
-   (or n (setq n (read-string (format "Binary word size: (default %d) "
-				      calc-word-size))))
+   (or n (setq n (read-string (format-prompt "Binary word size"
+                                             calc-word-size))))
    (setq n (if (stringp n)
 	       (if (equal n "")
 		   calc-word-size
@@ -145,9 +145,10 @@
    (setq math-half-2-word-size (math-power-of-2 (1- (math-abs n))))
    (calc-do-refresh)
    (calc-refresh-evaltos)
-   (if (< n 0)
-       (message "Binary word size is %d bits (two's complement)" (- n))
-     (message "Binary word size is %d bits" n))))
+   (cond
+    ((< n 0) (message "Binary word size is %d bits (two's complement)" (- n)))
+    ((> n 0) (message "Binary word size is %d bits" n))
+    (t (message "No fixed binary word size")))))
 
 
 
@@ -198,48 +199,16 @@
      (message "Omitting leading zeros on integers"))))
 
 
-(defvar math-power-of-2-cache (list 1 2 4 8 16 32 64 128 256 512 1024))
-(defvar math-big-power-of-2-cache nil)
 (defun math-power-of-2 (n)    ;  [I I] [Public]
-  (if (and (natnump n) (<= n 100))
-      (or (nth n math-power-of-2-cache)
-	  (let* ((i (length math-power-of-2-cache))
-		 (val (nth (1- i) math-power-of-2-cache)))
-	    (while (<= i n)
-	      (setq val (math-mul val 2)
-		    math-power-of-2-cache (nconc math-power-of-2-cache
-						 (list val))
-		    i (1+ i)))
-	    val))
-    (let ((found (assq n math-big-power-of-2-cache)))
-      (if found
-	  (cdr found)
-	(let ((po2 (math-ipow 2 n)))
-	  (setq math-big-power-of-2-cache
-		(cons (cons n po2) math-big-power-of-2-cache))
-	  po2)))))
+  (if (natnump n)
+      (ash 1 n)
+    (error "Argument must be a natural number")))
 
 (defun math-integer-log2 (n)    ; [I I] [Public]
-  (let ((i 0)
-	(p math-power-of-2-cache)
-	val)
-    (while (and p (Math-natnum-lessp (setq val (car p)) n))
-      (setq p (cdr p)
-	    i (1+ i)))
-    (if p
-	(and (equal val n)
-	     i)
-      (while (Math-natnum-lessp
-	      (prog1
-		  (setq val (math-mul val 2))
-		(setq math-power-of-2-cache (nconc math-power-of-2-cache
-						   (list val))))
-	      n)
-	(setq i (1+ i)))
-      (and (equal val n)
-	   i))))
-
-
+  (and (natnump n)
+       (not (zerop n))
+       (zerop (logand n (1- n)))
+       (logb n)))
 
 
 ;;; Bitwise operations.
@@ -262,9 +231,10 @@
 (defun math-binary-arg (a w)
   (if (not (Math-integerp a))
       (setq a (math-trunc a)))
-  (if (< a 0)
-      (logand a (1- (ash 1 (if w (math-trunc w) calc-word-size))))
-    a))
+  (let ((w (if w (math-trunc w) calc-word-size)))
+    (if (and (< a 0) (not (zerop w)))
+        (logand a (1- (ash 1 w)))
+      a)))
 
 (defun math-binary-modulo-args (f a b w)
   (let (mod)
@@ -285,7 +255,7 @@
     (let ((bits (math-integer-log2 mod)))
       (if bits
 	  (if w
-	      (if (/= w bits)
+	      (if (and (/= w bits) (not (zerop w)))
 		  (calc-record-why
 		   "*Warning: Modulus inconsistent with word size"))
 	    (setq w bits))
@@ -371,11 +341,12 @@
 	(math-clip (calcFunc-lsh a n (- w)) w)
       (if (Math-integer-negp a)
 	  (setq a (math-clip a w)))
-      (cond ((or (Math-lessp n (- w))
-		 (Math-lessp w n))
+      (cond ((and (or (Math-lessp n (- w))
+		      (Math-lessp w n))
+                  (not (zerop w)))
 	     0)
 	    ((< n 0)
-	     (math-quotient (math-clip a w) (math-power-of-2 (- n))))
+	     (ash (math-clip a w) n))
 	    (t
 	     (math-clip (math-mul a (math-power-of-2 n)) w))))))
 
@@ -401,9 +372,10 @@
 	  (math-clip (calcFunc-ash a n (- w)) w)
 	(if (Math-integer-negp a)
 	    (setq a (math-clip a w)))
-	(let ((two-to-sizem1 (math-power-of-2 (1- w)))
+	(let ((two-to-sizem1 (and (not (zerop w)) (math-power-of-2 (1- w))))
 	      (sh (calcFunc-lsh a n w)))
-	  (cond ((Math-natnum-lessp a two-to-sizem1)
+	  (cond ((or (zerop w)
+                     (zerop (logand a two-to-sizem1)))
 		 sh)
 		((Math-lessp n (- 1 w))
 		 (math-add (math-mul two-to-sizem1 2) -1))
@@ -421,6 +393,8 @@
   (if (eq (car-safe a) 'mod)
       (math-binary-modulo-args 'calcFunc-rot a n w)
     (setq w (if w (math-trunc w) calc-word-size))
+    (when (zerop w)
+      (error "Rotation requires a nonzero word size"))
     (or (integerp w)
 	(math-reject-arg w 'fixnump))
     (or (Math-integerp a)
@@ -432,7 +406,7 @@
       (if (Math-integer-negp a)
 	  (setq a (math-clip a w)))
       (cond ((or (Math-integer-negp n)
-		 (not (Math-natnum-lessp n w)))
+		 (>= n w))
 	     (calcFunc-rot a (math-mod n w) w))
 	    (t
 	     (math-add (calcFunc-lsh a (- n w) w)
@@ -449,9 +423,11 @@
 	 (math-reject-arg a 'integerp))
 	((< (or w (setq w calc-word-size)) 0)
 	 (setq a (math-clip a (- w)))
-	 (if (Math-natnum-lessp a (math-power-of-2 (- -1 w)))
+	 (if (< a (math-power-of-2 (- -1 w)))
 	     a
 	   (math-sub a (math-power-of-2 (- w)))))
+        ((math-zerop w)
+         a)
 	((Math-negp a)
 	 (math-binary-arg a w))
 	((integerp a)
@@ -682,6 +658,8 @@
 
 (defun math-format-twos-complement (a)
   "Format an integer in two's complement mode."
+  (when (zerop calc-word-size)
+    (error "Nonzero word size required"))
   (let* (;(calc-leading-zeros t)
          (num
           (cond
