@@ -1,6 +1,6 @@
-;;; calc-menu.el --- a menu for Calc
+;;; calc-menu.el --- a menu for Calc  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2007-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2007-2023 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -19,6 +19,8 @@
 
 ;;; Commentary:
 
+;;; Code:
+
 (defvar calc-arithmetic-menu
   (list "Arithmetic"
         (list "Basic"
@@ -781,7 +783,7 @@
                :active (>= (calc-stack-size) 2)
                :help "The cross product in R^3"]
               ["(2:) dot (1:)"
-               calc-mult
+               calc-times
                :keys "*"
                :active (>= (calc-stack-size) 2)
                :help "The dot product"]
@@ -1669,3 +1671,5 @@
         ["Quit" calc-quit]))
 
 (provide 'calc-menu)
+
+;;; calc-menu.el ends here
