@@ -1,6 +1,6 @@
-;;; hebrew.el --- support for Hebrew -*- coding: utf-8 -*-
+;;; hebrew.el --- support for Hebrew -*- coding: utf-8; lexical-binding: t; -*-
 
-;; Copyright (C) 2001-2020 Free Software Foundation, Inc.
+;; Copyright (C) 2001-2023 Free Software Foundation, Inc.
 ;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
 ;;   2005, 2006, 2007, 2008, 2009, 2010, 2011
 ;;   National Institute of Advanced Industrial Science and Technology (AIST)
@@ -240,14 +240,14 @@ Bidirectional editing is supported.")))
 
 (let* ((base "[\u05D0-\u05F2\uFB1D\uFB1F-\uFB28\uFB2A-\uFB4F]")
        (combining
-        "[\u0591-\u05BD\u05BF\u05C1-\u05C2\u05C4-\u05C5\u05C7\uFB1E]+")
+        "[\u034F\u0591-\u05BD\u05BF\u05C1-\u05C2\u05C4-\u05C5\u05C7\uFB1E]+")
        (pattern1 (concat base combining))
        (pattern2 (concat base "\u200D" combining)))
   (set-char-table-range
    composition-function-table '(#x591 . #x5C7)
-   (list (vector pattern2 3 'hebrew-shape-gstring)
-	 (vector pattern2 2 'hebrew-shape-gstring)
-	 (vector pattern1 1 'hebrew-shape-gstring)
+   (list (vector pattern2 3 #'hebrew-shape-gstring)
+	 (vector pattern2 2 #'hebrew-shape-gstring)
+	 (vector pattern1 1 #'hebrew-shape-gstring)
 	 [nil 0 hebrew-shape-gstring]))
   ;; Exclude non-combining characters.
   (set-char-table-range
