@@ -1,6 +1,6 @@
 ;;; xsd-regexp.el --- translate W3C XML Schema regexps to Emacs regexps  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2003, 2007-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2003, 2007-2023 Free Software Foundation, Inc.
 
 ;; Author: James Clark
 ;; Keywords: wp, hypermedia, languages, XML, regexp
@@ -24,7 +24,7 @@
 
 ;; This handles the regular expressions in the syntax defined by:
 ;; W3C XML Schema Part 2: Datatypes
-;; <http://www.w3.org/TR/xmlschema-2/#regexs>
+;; <https://www.w3.org/TR/xmlschema-2/#regexs>
 ;;
 ;; The main entry point is `xsdre-translate'.
 ;;
@@ -52,9 +52,6 @@
 ;; or a character translatable to such a character (i.e a character
 ;; for which `encode-char' will return non-nil).
 ;;
-;; Using unify-8859-on-decoding-mode is probably a good idea here
-;; (and generally with XML and other Unicode-oriented formats).
-;;
 ;; Unfortunately, this means that this package is currently useless
 ;; for CJK characters, since there's no mule-unicode charset for the
 ;; CJK ranges of Unicode.  We should devise a workaround for this
@@ -290,7 +287,7 @@ and whose tail is ACCUM."
 (defun xsdre-compile-single-char (ch)
   (if (memq ch '(?. ?* ?+ ?? ?\[ ?\] ?^ ?$ ?\\))
       (string ?\\ ch)
-    (string (decode-char 'ucs ch))))
+    (string ch)))
 
 (defun xsdre-char-class-to-range-list (cc)
   "Return a range-list for a symbolic char-class CC."
@@ -387,9 +384,15 @@ consisting of a single char alternative delimited with []."
 			((eq first ?-)
 			 (setq hyphen t)
 			 (setq first (1+ first)))
+			((eq last ?-)
+			 (setq hyphen t)
+			 (setq last (1- last)))
 			((eq first ?\])
 			 (setq close-bracket t)
-			 (setq first (1+ first))))
+			 (setq first (1+ first)))
+			((eq last ?\])
+			 (setq close-bracket t)
+			 (setq last (1- last))))
 		  (<= first last)))
       (when (<= first last)
 	(setq chars
@@ -401,10 +404,6 @@ consisting of a single char alternative delimited with []."
 		    (cons last chars)
 		  (cons last (cons ?- chars))))))
       (setq range-list (cdr range-list)))
-    (setq chars
-	  (mapcar (lambda (c)
-		    (decode-char 'ucs c))
-		  chars))
     (when caret
       (setq chars (cons ?^ chars)))
     (when hyphen
@@ -702,7 +701,7 @@ Code is inserted into the current buffer."
   (with-current-buffer (find-file-noselect file)
     (goto-char (point-min))
     (mapc (lambda (x) (put x 'xsdre-ranges nil)) xsdre-gen-categories)
-    (while (re-search-forward "^\\([0-9A-Fa-f]*\\);[^;]*;\\([A-Z][a-z]\\);"
+    (while (re-search-forward "^\\([[:xdigit:]]*\\);[^;]*;\\([A-Z][a-z]\\);"
 			      nil
 			      t)
       (let* ((sym (intern (match-string-no-properties 2)))
@@ -1219,7 +1218,7 @@ Code is inserted into the current buffer."
 
 ;; The rest of the file was auto-generated by doing M-x xsdre-gen-categories
 ;; on UnicodeData-3.1.0.txt available from
-;; http://www.unicode.org/Public/3.1-Update/UnicodeData-3.1.0.txt
+;; https://www.unicode.org/Public/3.1-Update/UnicodeData-3.1.0.txt
 
 (xsdre-def-primitive-category 'Lu
 			      '((65 . 90)
