@@ -1,5 +1,5 @@
 /* Input event support for Emacs on the Microsoft Windows API.
-   Copyright (C) 1992-1993, 1995, 2001-2021 Free Software Foundation,
+   Copyright (C) 1992-1993, 1995, 2001-2023 Free Software Foundation,
    Inc.
 
 This file is part of GNU Emacs.
@@ -420,7 +420,7 @@ w32_console_mouse_position (struct frame **f,
   *f = get_frame ();
   *bar_window = Qnil;
   *part = scroll_bar_above_handle;
-  SELECTED_FRAME ()->mouse_moved = 0;
+  (*f)->mouse_moved = 0;
 
   XSETINT (*x, movement_pos.X);
   XSETINT (*y, movement_pos.Y);
@@ -436,7 +436,8 @@ mouse_moved_to (int x, int y)
   /* If we're in the same place, ignore it.  */
   if (x != movement_pos.X || y != movement_pos.Y)
     {
-      SELECTED_FRAME ()->mouse_moved = 1;
+      struct frame *f = get_frame ();
+      f->mouse_moved = 1;
       movement_pos.X = x;
       movement_pos.Y = y;
       movement_time = GetTickCount ();
@@ -471,13 +472,13 @@ do_mouse_event (MOUSE_EVENT_RECORD *event,
   int i;
 
   /* Mouse didn't move unless MOUSE_MOVED says it did.  */
-  SELECTED_FRAME ()->mouse_moved = 0;
+  struct frame *f = get_frame ();
+  f->mouse_moved = 0;
 
   switch (flags)
     {
     case MOUSE_MOVED:
       {
-	struct frame *f = get_frame ();
 	Mouse_HLInfo *hlinfo = MOUSE_HL_INFO (f);
 	int mx = event->dwMousePosition.X, my = event->dwMousePosition.Y;
 
@@ -536,7 +537,6 @@ do_mouse_event (MOUSE_EVENT_RECORD *event,
     case MOUSE_WHEELED:
     case MOUSE_HWHEELED:
       {
-	struct frame *f = get_frame ();
 	/* Mouse positions in console wheel events are reported to
 	   ReadConsoleInput relative to the display's top-left
 	   corner(!), not relative to the origin of the console screen
@@ -588,8 +588,8 @@ do_mouse_event (MOUSE_EVENT_RECORD *event,
 
       int x = event->dwMousePosition.X;
       int y = event->dwMousePosition.Y;
-      struct frame *f = get_frame ();
-      emacs_ev->arg = tty_handle_tab_bar_click (f, x, y, (button_state & mask) != 0,
+      emacs_ev->arg = tty_handle_tab_bar_click (f, x, y,
+						(button_state & mask) != 0,
 						emacs_ev);
 
       emacs_ev->modifiers |= ((button_state & mask)
@@ -648,7 +648,7 @@ handle_file_notifications (struct input_event *hold_quit)
       ns = NULL;
 
       /* Find out if there is a record available in the linked list of
-	 notifications sets.  If so, unlink te set from the linked list.
+	 notifications sets.  If so, unlink the set from the linked list.
 	 Use the critical section.  */
       enter_crit ();
       if (notifications_set_head->next != notifications_set_head)
