@@ -1,6 +1,6 @@
 ;;; copyright-tests.el --- tests for copyright.el  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2020-2022 Free Software Foundation, Inc.
+;; Copyright (C) 2020-2023 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -59,7 +59,8 @@
                 "\nCopyright 2006, 2007, 2008 Foo Bar\n\n")
         (copyright-update)
         (buffer-substring (- (point-max) 42) (point-max))))
-    "Copyright 2006, 2007, 2008, 2022 Foo Bar\n\n")))
+    (format "Copyright 2006, 2007, 2008, %s Foo Bar\n\n"
+            (format-time-string "%Y")))))
 
 (ert-deftest test-correct-notice ()
   (should (equal
@@ -70,7 +71,28 @@
                    (copyright-query nil))
                (copyright-update))
              (buffer-string))
-           "Copyright 2021 FSF\nCopyright 2021, 2022 FSF\n")))
+           (format "Copyright 2021 FSF\nCopyright 2021, %s FSF\n"
+                   (format-time-string "%Y")))))
+
+(defmacro with-copyright-fix-years-test (orig result)
+  `(let ((copyright-year-ranges t))
+     (with-temp-buffer
+       (insert ,orig)
+       (copyright-fix-years)
+       (should (equal (buffer-string) ,result)))))
+
+(defvar copyright-fix-years-tests--data
+  '((";; Copyright (C) 2008, 2010, 2012"
+     . ";; Copyright (C) 2008, 2010, 2012")
+    (";; Copyright (C) 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018"
+     . ";; Copyright (C) 2008-2010, 2013-2016, 2018")
+    (";; Copyright (C) 2008-2010, 2011, 2015, 2016, 2017"
+     . ";; Copyright (C) 2008-2010, 2011, 2015-2017")))
+
+(ert-deftest text-copyright-fix-years ()
+  "Test basics of \\[copyright-fix-years]."
+  (dolist (test copyright-fix-years-tests--data)
+    (with-copyright-fix-years-test (car test) (cdr test))))
 
 (provide 'copyright-tests)
 ;;; copyright-tests.el ends here
