@@ -1,6 +1,6 @@
 ;;; css-mode-tests.el --- Test suite for CSS mode  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016-2022 Free Software Foundation, Inc.
+;; Copyright (C) 2016-2023 Free Software Foundation, Inc.
 
 ;; Author: Simen Heggestøyl <simenheg@gmail.com>
 ;; Keywords: internal
@@ -419,5 +419,74 @@
       (indent-region (point-min) (point-max))
       (should (equal (buffer-string) orig)))))
 
+(ert-deftest css-mode-test-selectors ()
+  (let ((selectors
+         (with-temp-buffer
+           (insert-file-contents (ert-resource-file "css-selectors.txt"))
+           (string-lines (buffer-string)))))
+    (with-suppressed-warnings ((interactive-only font-lock-debug-fontify))
+      (dolist (selector selectors)
+        (with-temp-buffer
+          (css-mode)
+          (insert selector " {\n}\n")
+          (font-lock-debug-fontify)
+          (goto-char (point-min))
+          (unless (eq (get-text-property (point) 'face)
+                      'css-selector)
+            (should-not (format "Didn't recognize %s as a selector"
+                                (buffer-substring-no-properties
+                                 (point) (pos-eol)))))))
+      ;; Test many selectors.
+      (dolist (selector selectors)
+        (with-temp-buffer
+          (css-mode)
+          (insert selector " ")
+          (dotimes (_ (random 5))
+            (insert (seq-random-elt '(" , " " > " " + "))
+                    (seq-random-elt selectors)))
+          (insert "{\n}\n")
+          (font-lock-debug-fontify)
+          (goto-char (point-min))
+          (unless (eq (get-text-property (point) 'face)
+                      'css-selector)
+            (should-not (format "Didn't recognize %s as a selector"
+                                (buffer-substring-no-properties
+                                 (point) (pos-eol)))))))
+      ;; Test wrong separators.
+      (dolist (selector selectors)
+        (with-temp-buffer
+          (css-mode)
+          (insert selector " ")
+          (dotimes (_ (1+ (random 5)))
+            (insert (seq-random-elt '("=" " @ "))
+                    (seq-random-elt selectors)))
+          (insert "{\n}\n")
+          (font-lock-debug-fontify)
+          (goto-char (point-min))
+          (when (eq (get-text-property (point) 'face)
+                    'css-selector)
+            (should-not (format "Recognized %s as a selector"
+                                (buffer-substring-no-properties
+                                 (point) (pos-eol))))))))))
+
+(ert-deftest scss-mode-test-selectors ()
+  (let ((selectors
+         (with-temp-buffer
+           (insert-file-contents (ert-resource-file "scss-selectors.txt"))
+           (string-lines (buffer-string)))))
+    (with-suppressed-warnings ((interactive-only font-lock-debug-fontify))
+      (dolist (selector selectors)
+        (with-temp-buffer
+          (scss-mode)
+          (insert selector " {\n}\n")
+          (font-lock-debug-fontify)
+          (goto-char (point-min))
+          (unless (eq (get-text-property (point) 'face)
+                      'css-selector)
+            (should-not (format "Didn't recognize %s as a selector"
+                                (buffer-substring-no-properties
+                                 (point) (pos-eol))))))))))
+
+
 (provide 'css-mode-tests)
 ;;; css-mode-tests.el ends here
